#ifndef FGGL_ECS_COMPONENT_H
#define FGGL_ECS_COMPONENT_H

#include "utility.hpp"

#include <new>
#include <utility>

#include <vector>
#include <unordered_map>

namespace fggl::ecs {

	class ComponentBase {
		public:
			using data_t = unsigned char;
			virtual ~ComponentBase() {};

			virtual void destroy(data_t* data) const = 0;
			virtual void move(data_t* src, data_t* dest) const = 0;
			virtual void construct(data_t* data) const = 0;

			virtual std::size_t size() const = 0;
	};

	template<class C>
	class Component : public ComponentBase {
		public:
			virtual void destroy(data_t* data) const override {
				C* location = std::launder(reinterpret_cast<C*>(data));
				location->~C();
			}

			virtual void construct(unsigned char* data) const override {
//				new (&data[0]) C();
			}

			virtual void move(data_t* src, data_t* dest) const override {
				new (&dest[0]) C(std::move(*reinterpret_cast<C*>(src)));
			}

			virtual std::size_t size() const {
				return sizeof(C);
			}

			static component_type_t typeID() {
				return TypeIdGenerator<ComponentBase>::GetNewID<C>();
			}
	};

}

#endif
